//#include <stdio.h>
//#include <unistd.h>
//#include <stdlib.h>
#include <string.h>
//#include <getopt.h>

#include "piControlIf.h"
#include "piControl.h"
#include "modfuns.h"

char *getWriteError(int error);

// Schreiben einer Variable im Prozessabbild
// Veränderungen: 	- Entfernen der printfs bei Erfolg
// 					- gibt 0 bei Erfolg zurück, 1 bei Fehler

int mod_writeVariableValue(char *pszVariableName, uint32_t i32uValue){
	int rc;
	SPIVariable sPiVariable;
	SPIValue sPIValue;
	uint8_t i8uValue;
	uint16_t i16uValue;

	strncpy(sPiVariable.strVarName, pszVariableName, sizeof(sPiVariable.strVarName));
	rc = piControlGetVariableInfo(&sPiVariable);
	if (rc < 0) {return 1;}

	if (sPiVariable.i16uLength == 1) {
		sPIValue.i16uAddress = sPiVariable.i16uAddress;
		sPIValue.i8uBit = sPiVariable.i8uBit;
		sPIValue.i8uValue = i32uValue;
		rc = piControlSetBitValue(&sPIValue);
		if (rc < 0) {return 1;}
	} else if (sPiVariable.i16uLength == 8) {
		i8uValue = i32uValue;
		rc = piControlWrite(sPiVariable.i16uAddress, 1, (uint8_t *) & i8uValue);
		if (rc < 0){ return 1;}
	} else if (sPiVariable.i16uLength == 16) {
		i16uValue = i32uValue;
		rc = piControlWrite(sPiVariable.i16uAddress, 2, (uint8_t *) & i16uValue);
		if (rc < 0){ return 1;}
	} else if (sPiVariable.i16uLength == 32) {
		rc = piControlWrite(sPiVariable.i16uAddress, 4, (uint8_t *) & i32uValue);
		if (rc < 0){ return 1;}
	}
	return 0;
}


// Lesen einer Variable im Prozessabbild
// Veränderungen: 	- Entfernen aller printfs bei Erfolg
//					- Rückgabe des Variablenwertes als int
//					- Entfernen von cyclic, format und quiet
// Gibt den Wert der Variablen zurück, oder, im Falle eines Fehlers, -1
int mod_readVariableValue(char *pszVariableName){
	int rc;
	SPIVariable sPiVariable;
	SPIValue sPIValue;
	uint8_t i8uValue;
	uint16_t i16uValue;
	uint32_t i32uValue;

	strncpy(sPiVariable.strVarName, pszVariableName, sizeof(sPiVariable.strVarName));
	rc = piControlGetVariableInfo(&sPiVariable);
	if (rc < 0) {return -1;}
	
	if (sPiVariable.i16uLength == 1) {
		sPIValue.i16uAddress = sPiVariable.i16uAddress;
		sPIValue.i8uBit = sPiVariable.i8uBit;
		rc = piControlGetBitValue(&sPIValue);
		if (rc < 0){}
		else {return sPIValue.i8uValue;}
	} 
	
	else if (sPiVariable.i16uLength == 8) {
		rc = piControlRead(sPiVariable.i16uAddress, 1, (uint8_t *) & i8uValue);
		if (rc < 0){ return -1;}
		else { return i8uValue;}
	}

	else if (sPiVariable.i16uLength == 16) {
		rc = piControlRead(sPiVariable.i16uAddress, 2, (uint8_t *) & i16uValue);
		if (rc < 0){return -1;}
		else { return i16uValue;}
	}


	else if (sPiVariable.i16uLength == 32) {
		rc = piControlRead(sPiVariable.i16uAddress, 4, (uint8_t *) & i32uValue);
		if (rc < 0){ return -1;}
		else { return i32uValue;}
	}
	
	return -1;
}


// Setzt die LEDs auf Grün, Orange oder Rot
void SetPiLED(char LED1, char LED2){
	int LED[2]; // Zustandsvariablen für die LEDs
	
	//Setzen der zustandsvariable für LED A1
	if(LED1==' '||LED1=='-'||LED1=='/'||LED1=='X'||LED1=='x'){LED[0]=0;}
	else if(LED1=='G'||LED1=='g'){LED[0]=1;}
	else if(LED1=='R'||LED1=='r'){LED[0]=2;}
	else if(LED1=='O'||LED1=='o'){LED[0]=3;}
	else {LED[0]=0;}
	
	//Setzen der zustandsvariable für LED A2
	if(LED2==' '||LED2=='-'||LED2=='/'||LED2=='X'||LED2=='x'){LED[1]=0;}
	else if(LED2=='G'||LED2=='g'){LED[1]=1;}
	else if(LED2=='R'||LED2=='r'){LED[1]=2;}
	else if(LED2=='O'||LED2=='o'){LED[1]=3;}
	else {LED[1]=0;}

	// Beschreiben des Ausganges:
	mod_writeVariableValue("RevPiLED",LED[0] + 4*LED[1]);
}